@@ -1,18 +1,18 @@
 /*
  * Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
  */
@@ -35,12 +35,11 @@ long long int llroundl(long double x);
 
 #include <gst/tag/tag.h>
 
-#include <libtracker-client/tracker.h>
-
+#include <libtracker-common/tracker-common.h>
+#include <libtracker-sparql/tracker-sparql.h>
 #include <libtracker-extract/tracker-extract.h>
 
 #include "tracker-albumart.h"
-#include "tracker-dbus.h"
 
 /* An additional tag in gstreamer for the content source. Remove when in upstream */
 #ifndef GST_TAG_CLASSIFICATION
@@ -117,46 +116,26 @@ typedef enum {
 } ContentType;
 
 typedef struct {
-	ContentType     content;
-	gboolean        has_image;
-	gboolean        has_audio;
-	gboolean        has_video;
-	const gchar    *dlna_profile;
-	GstTagList     *tags;
-	guint           width;
-	guint           height;
-	gfloat          frame_rate;
-	gfloat          aspect_ratio;
-	guint           sample_rate;
-	guint           bitrate;
-	guint           channels;
-	guint           duration;
-	gboolean        is_content_encrypted;
-	unsigned char  *album_art_data;
-	guint           album_art_size;
-	const gchar    *album_art_mime;
+	ContentType content;
+	gboolean has_image;
+	gboolean has_audio;
+	gboolean has_video;
+	const gchar *dlna_profile;
+	GstTagList *tags;
+	guint width;
+	guint height;
+	gfloat frame_rate;
+	gfloat aspect_ratio;
+	guint sample_rate;
+	guint bitrate;
+	guint channels;
+	guint duration;
+	gboolean is_content_encrypted;
+	unsigned char *album_art_data;
+	guint album_art_size;
+	const gchar *album_art_mime;
 } MetadataExtractor;
 
-static void
-add_int64_info (TrackerSparqlBuilder *metadata,
-                const gchar          *uri,
-                const gchar          *key,
-                gint64                info)
-{
-	tracker_sparql_builder_predicate (metadata, key);
-	tracker_sparql_builder_object_int64 (metadata, info);
-}
-
-static void
-add_uint_info (TrackerSparqlBuilder *metadata,
-               const gchar          *uri,
-               const gchar          *key,
-               guint                 info)
-{
-	tracker_sparql_builder_predicate (metadata, key);
-	tracker_sparql_builder_object_int64 (metadata, info);
-}
-
 static void
 add_string_gst_tag (TrackerSparqlBuilder *metadata,
                     const gchar          *uri,
@@ -197,24 +176,6 @@ add_uint_gst_tag (TrackerSparqlBuilder *metadata,
 	}
 }
 
-static void
-add_int_gst_tag (TrackerSparqlBuilder *metadata,
-                 const gchar          *uri,
-                 const gchar          *key,
-                 GstTagList           *tag_list,
-                 const gchar          *tag)
-{
-	gboolean ret;
-	gint n;
-
-	ret = gst_tag_list_get_int (tag_list, tag, &n);
-
-	if (ret) {
-		tracker_sparql_builder_predicate (metadata, key);
-		tracker_sparql_builder_object_int64 (metadata, n);
-	}
-}
-
 static void
 add_double_gst_tag (TrackerSparqlBuilder *metadata,
                     const gchar          *uri,
@@ -233,30 +194,6 @@ add_double_gst_tag (TrackerSparqlBuilder *metadata,
 	}
 }
 
-static void
-add_fraction_gst_tag (TrackerSparqlBuilder *metadata,
-                      const gchar          *uri,
-                      const gchar          *key,
-                      GstTagList           *tag_list,
-                      const gchar          *tag)
-{
-	gboolean ret;
-	GValue n = {0,};
-	gfloat f;
-
-	ret = gst_tag_list_copy_value (&n, tag_list, tag);
-
-	if (ret) {
-		f = (gfloat)gst_value_get_fraction_numerator (&n)/
-		gst_value_get_fraction_denominator (&n);
-
-		tracker_sparql_builder_predicate (metadata, key);
-		tracker_sparql_builder_object_double (metadata, (gdouble) f);
-
-		g_value_unset (&n);
-	}
-}
-
 static void
 add_y_date_gst_tag (TrackerSparqlBuilder  *metadata,
                     const gchar           *uri,
@@ -265,55 +202,39 @@ add_y_date_gst_tag (TrackerSparqlBuilder  *metadata,
                     const gchar           *tag)
 {
 	GDate *date;
-	gboolean ret;
+	gchar buf[25];
 
 	date = NULL;
-	ret = gst_tag_list_get_date (tag_list, tag, &date);
+	buf[0] = '\0';
+
+	if (gst_tag_list_get_date (tag_list, tag, &date)) {
+		gboolean ret;
 
-	if (ret) {
 		if (date && g_date_valid (date)) {
 			if (date->julian)
 				ret = g_date_valid_julian (date->julian_days);
 			if (date->dmy)
 				ret = g_date_valid_dmy (date->day, date->month, date->year);
-		} else
+		} else {
 			ret = FALSE;
-	}
+		}
 
-	if (ret) {
-		gchar buf[25];
-
-		if (g_date_strftime (buf, 25, "%Y-%m-%dT%H:%M:%S%z", date)) {
-			tracker_sparql_builder_predicate (metadata, key);
-			tracker_sparql_builder_object_unvalidated (metadata, buf);
+		if (ret) {
+			if (g_date_strftime (buf, 25, "%Y-%m-%dT%H:%M:%S%z", date)) {
+				tracker_sparql_builder_predicate (metadata, key);
+				tracker_sparql_builder_object_unvalidated (metadata, buf);
+			}
 		}
 	}
 
 	if (date) {
 		g_date_free (date);
 	}
-}
 
-static void
-add_time_gst_tag (TrackerSparqlBuilder *metadata,
-                  const gchar          *uri,
-                  const gchar          *key,
-                  GstTagList           *tag_list,
-                  const gchar          *tag)
-{
-	gboolean ret;
-	guint64 n;
-
-	ret = gst_tag_list_get_uint64 (tag_list, tag, &n);
-
-	if (ret) {
-		gint64 duration;
-
-		duration = llroundl ((long double) n / (long double) GST_SECOND);
-
-		tracker_sparql_builder_predicate (metadata, key);
-		tracker_sparql_builder_object_int64 (metadata, duration);
-	}
+	tracker_guarantee_date_from_file_mtime (metadata,
+	                                        key,
+	                                        buf,
+	                                        uri);
 }
 
 static void
@@ -366,6 +287,7 @@ extract_metadata (MetadataExtractor      *extractor,
 		gchar *composer_uri = NULL;
 		gchar *album_uri = NULL;
 		gchar *album_disc_uri = NULL;
+		gchar *s;
 
 		/* General */
 		if (extractor->content == CONTENT_AUDIO || extractor->content == CONTENT_VIDEO) {
@@ -577,8 +499,9 @@ extract_metadata (MetadataExtractor      *extractor,
 
 		}
 
-		if (extractor->content == CONTENT_AUDIO)
+		if (extractor->content == CONTENT_AUDIO) {
 			needs_audio = TRUE;
+		}
 
 		tracker_sparql_builder_predicate (metadata, "a");
 
@@ -594,13 +517,20 @@ extract_metadata (MetadataExtractor      *extractor,
 		}
 
 		gst_tag_list_get_string (extractor->tags, GST_TAG_GENRE, &genre);
-		if (g_strcmp0 (genre, "Unknown") != 0) {
+		if (genre && g_strcmp0 (genre, "Unknown") != 0) {
 			tracker_sparql_builder_predicate (metadata, "nfo:genre");
 			tracker_sparql_builder_object_unvalidated (metadata, genre);
 		}
 		g_free (genre);
 
-		add_string_gst_tag (metadata, uri, "nie:title", extractor->tags, GST_TAG_TITLE);
+		s = NULL;
+		ret = gst_tag_list_get_string (extractor->tags, GST_TAG_TITLE, &s);
+		tracker_guarantee_title_from_file (metadata,
+		                                   "nie:title",
+		                                   s,
+		                                   uri);
+		g_free (s);
+
 		add_string_gst_tag (metadata, uri, "nie:copyright", extractor->tags, GST_TAG_COPYRIGHT);
 		add_string_gst_tag (metadata, uri, "nie:license", extractor->tags, GST_TAG_LICENSE);
 		add_string_gst_tag (metadata, uri, "dc:coverage", extractor->tags, GST_TAG_LOCATION);
@@ -627,10 +557,12 @@ extract_metadata (MetadataExtractor      *extractor,
 				tracker_sparql_builder_predicate (preupdate, "nfo:manufacturer");
 				tracker_sparql_builder_object_unvalidated (preupdate, manuf ? manuf : make);
 			}
+
 			if (model) {
 				tracker_sparql_builder_predicate (preupdate, "nfo:model");
 				tracker_sparql_builder_object_unvalidated (preupdate, model);
 			}
+
 			tracker_sparql_builder_insert_close (preupdate);
 
 			tracker_sparql_builder_predicate (metadata, "nfo:equipment");
@@ -767,15 +699,15 @@ extract_metadata (MetadataExtractor      *extractor,
 }
 
 static void
-extract_gupnp_dlna (const gchar           *uri,
-                    TrackerSparqlBuilder  *preupdate,
-                    TrackerSparqlBuilder  *metadata,
-                    ContentType            content)
+extract_gupnp_dlna (const gchar          *uri,
+                    TrackerSparqlBuilder *preupdate,
+                    TrackerSparqlBuilder *metadata,
+                    ContentType           content)
 {
-	GUPnPDLNADiscoverer  *discoverer = NULL;
-	GUPnPDLNAInformation *dlna_info  = NULL;
-	GError               *error      = NULL;
-	MetadataExtractor    extractor;
+	GUPnPDLNADiscoverer *discoverer;
+	GUPnPDLNAInformation *dlna_info;
+	GError *error = NULL;
+	MetadataExtractor extractor;
 
 	gst_init (NULL, NULL);
 
@@ -783,7 +715,7 @@ extract_gupnp_dlna (const gchar           *uri,
 	extractor.has_video = FALSE;
 	extractor.has_audio = FALSE;
 
-	extractor.tags      = NULL;
+	extractor.tags = NULL;
 
 	extractor.width = 0;
 	extractor.height = 0;
@@ -802,76 +734,82 @@ extract_gupnp_dlna (const gchar           *uri,
 	extractor.album_art_size = 0;
 	extractor.album_art_mime = NULL;
 
-	discoverer = gupnp_dlna_discoverer_new (5*GST_SECOND, TRUE, FALSE);
+	discoverer = gupnp_dlna_discoverer_new (5 * GST_SECOND, TRUE, FALSE);
+
+	/* Uri is const, the API should be const, but it isn't and it
+	 * calls gst_discoverer_discover_uri()
+	 */
 	dlna_info = gupnp_dlna_discoverer_discover_uri_sync (discoverer,
 							     uri,
 							     &error);
 
 	if (error) {
-		g_warning ("Error in gst-discovery: %s", error->message);
-
+		g_warning ("Call to gupnp_dlna_discoverer_discover_uri_sync() failed: %s", error->message);
 		g_error_free (error);
 		return;
 	}
 
 	if (dlna_info) {
-		GstDiscovererInformation *info     = NULL;
-		GList                    *iter;
-
+		GstDiscovererInfo *info;
+		GList *streams;
+		GList *iter;
 		gchar *artist, *album, *scount;
 
-		album  = NULL;
+		album = NULL;
 		scount = NULL;
 		artist = NULL;
 
-		info = gupnp_dlna_information_get_info (dlna_info);
+		info = (GstDiscovererInfo*) gupnp_dlna_information_get_info (dlna_info);
 
-		iter = g_list_first (info->stream_list);
+		streams = iter = gst_discoverer_info_get_stream_list (info);
 
 		while (iter != NULL) {
-			GstStreamInformation *stream   = NULL;
-			GstTagList           *tmp      = NULL;
+			GstDiscovererStreamInfo *stream;
+			GstTagList *tmp;
 
 			stream = iter->data;
 
-			if (stream->misc == NULL) {
-				g_debug ("misc NULL");
-			}
+			if (G_TYPE_CHECK_INSTANCE_TYPE (stream, GST_TYPE_DISCOVERER_AUDIO_INFO)) {
+				GstDiscovererAudioInfo *audio = (GstDiscovererAudioInfo*)stream;
 
-			switch (stream->streamtype) {
-			case GST_STREAM_AUDIO:
 				extractor.has_audio = TRUE;
-				extractor.sample_rate = ((GstStreamAudioInformation *)stream)->sample_rate;
-				extractor.bitrate     = ((GstStreamAudioInformation *)stream)->bitrate;
-				extractor.channels    = ((GstStreamAudioInformation *)stream)->channels;
-				break;
-			case GST_STREAM_VIDEO:
-				extractor.has_video    = TRUE;
-				extractor.frame_rate   = (gfloat)gst_value_get_fraction_numerator (&((GstStreamVideoInformation *)stream)->frame_rate)/
-				                            gst_value_get_fraction_denominator (&((GstStreamVideoInformation *)stream)->frame_rate);
-				extractor.width        = ((GstStreamVideoInformation *)stream)->width;
-				extractor.height       = ((GstStreamVideoInformation *)stream)->height;
-				extractor.aspect_ratio = (gfloat)gst_value_get_fraction_numerator (&((GstStreamVideoInformation *)stream)->pixel_aspect_ratio)/
-				                            gst_value_get_fraction_denominator (&((GstStreamVideoInformation *)stream)->pixel_aspect_ratio);
-				break;
-			case GST_STREAM_IMAGE:
-				extractor.has_image = TRUE;
-				break;
-			default:
-				break;
+				extractor.sample_rate = gst_discoverer_audio_info_get_sample_rate (audio);
+				extractor.bitrate = gst_discoverer_audio_info_get_bitrate (audio);
+				extractor.channels = gst_discoverer_audio_info_get_channels (audio);
+			} else if (G_TYPE_CHECK_INSTANCE_TYPE (stream, GST_TYPE_DISCOVERER_VIDEO_INFO)) {
+				GstDiscovererVideoInfo *video = (GstDiscovererVideoInfo*) stream;
+
+				if (gst_discoverer_video_info_is_image (video)) {
+					extractor.has_image = TRUE;
+				} else {
+					extractor.has_video = TRUE;
+					extractor.frame_rate = (gfloat)
+						gst_discoverer_video_info_get_framerate_num (video) /
+						gst_discoverer_video_info_get_framerate_denom (video);
+					extractor.width = gst_discoverer_video_info_get_width (video);
+					extractor.height = gst_discoverer_video_info_get_height (video);
+					extractor.aspect_ratio = (gfloat)
+						gst_discoverer_video_info_get_par_num (video) /
+						gst_discoverer_video_info_get_par_denom (video);
+				}
+			} else {
+				/* Unknown type - do nothing */
 			}
 
-			tmp = gst_tag_list_merge (extractor.tags, stream->tags, GST_TAG_MERGE_APPEND);
+			tmp = gst_tag_list_merge (extractor.tags,
+			                          gst_discoverer_stream_info_get_tags (stream),
+			                          GST_TAG_MERGE_APPEND);
+
 			if (extractor.tags) {
 				gst_tag_list_free (extractor.tags);
 			}
+
 			extractor.tags = tmp;
 
 			iter = g_list_next (iter);
 		}
 
-		extractor.duration = info->duration / GST_SECOND;
-
+		extractor.duration = gst_discoverer_info_get_duration (info) / GST_SECOND;
 		extractor.dlna_profile = gupnp_dlna_information_get_name (dlna_info);
 
 		if (content != CONTENT_GUESS) {
@@ -891,10 +829,10 @@ extract_gupnp_dlna (const gchar           *uri,
 		extract_metadata (&extractor, uri, preupdate, metadata, &artist, &album, &scount);
 
 		gst_tag_list_free (extractor.tags);
+		gst_discoverer_stream_info_list_free (streams);
 		g_free (artist);
 		g_free (album);
 		g_free (scount);
-
 	}
 
 	g_object_unref (discoverer);
@@ -939,4 +877,3 @@ tracker_extract_get_data (void)
 {
 	return data;
 }
-
